<?php


/**
 * File containing default profile system
 * @package account_profile
 */

/**
 * @ignore
 */
template::setInlineCss(config::getModulePath('account_profile') . '/assets/account_profile.css');


//print_r(layout::$menu);

/**
 * default account profiles main class. 
 * @package account_profile
 */
class account_profile {

    /**
     *
     * @var array holding errors
     */
    public static $errors = array ();

    /**
     * method for getting userid when profile page
     * @return int  $id
     */
    public static function getId (){
        $uri = uri::getInstance();
        return $uri->fragment(2);
    }

    /**
     * default method for getting profile info
     * @param array $user info
     * @param  string $text string to add to profile link
     * @return string  $profile as a html string
     */
    public static function getProfile ($user, $text){

        if (config::getModuleIni('account_profile_simple')) {
            return self::getProfileSimple($user, $text);
        }
        
        //var_dump($user);
	if (!isset($user) || empty($user)){
            return '';
        }

        if (!is_array($user)){
            return '';
        }

        // for anonymous user
        if ($user['id'] === 0) {
            return self::createIdenticonTableAnon($user, $text);
        } else {
            return self::createIdenticonTable($user, $text);
        }       
    }
    
        /**
     * default method for getting profile info
     * @param array $user info
     * @param  string $text string to add to profile link
     * @return string  $profile as a html string
     */
    public static function getProfileAnon ($user, $text){
        return self::createIdenticonTableAnon($user, $text);
 
    }
    
    /**
     * get logout html based on profile
     * @param array $row account row
     * @return string $html 
     */
    public static function getLogoutHTML ($row) {

        $logout_url = "/account/login/index/1";
        $str = '';

        $str.= lang::translate('account_profile_logged_in_with_email');
        $str.= MENU_SUB_SEPARATOR_SEC . "<br />" . $row['email'] . ' ';
        $str.= "(" . lang::translate('account_profile_login_will_not_be_displayed') . ")";
        $str.= '<br />';
        
        $profile_info = user::getProfileInfo($row);

        if (empty($profile_info['screenname'])) {
            $str.= lang::translate('account_profile_no_screenname');
        } else {
            $str.= lang::translate('account_profile_your_screenname_is') . MENU_SUB_SEPARATOR_SEC . $profile_info['screenname'];
        }

        $str.='<br />';
        $str.= user::getProfileEditLink($row['id']);
        $str.= "<hr />" . html::createLink($logout_url, lang::translate('account_profile_logout'));        
        return $str;
    } 

    
    /**
     * default method for getting profile info
     * @param array $user info
     * @param  string $text string to add to profile link
     * @return string  $profile as a html string
     */
    public static function getProfileSimple ($user, $text){

        if (!isset($user)){
            return '';
        }

        if (empty($user)){
            return '';
        }
        
        // for anonymous user
        if ($user['id'] === 0) {
            return self::getSimpleAnonLink($user, $text);
        } else {
            return self::getSimpleUserLink($user, $text);
        }       
    }

    /**
     * return profile as a table block with image, use link, and
     * info
     *
     * @param  array $user 
     *               array ('homepage' => 'http://example.com',
     *                      'name' => 'example name');  
     * @param  array $options
     * @return string
     */
    public static function getSimpleAnonLink ($user, $text = ''){
        $str = '';
        if (!empty($user['homepage'])){
            $str.= html::createLink($user['homepage'], $user['name']);
        } else {
            $str.= $user['name'];
        }
        if (!empty($text)){
            $str.= " ($text) ";
        }
        return $str;
    }
    
    
    /**
     * @param  array $user  
     * @param  array $text
     * @return string
     */
    public static function getSimpleUserLink ($user, $text = ''){
        $str = self::getProfileLink($user['id']);
        if (!empty($text)){
            $str.= " ($text) ";
        }
        return $str;
    }
    


    /**
     * returns profile as a link
     * @param   int     $id
     * @return  string  $str html profile link
     */
    public static function getProfileLink ($id){
        $profile = self::getAccountProfileFromId($id);
        if (!empty($profile['screenname'])){
            $screenname = $profile['screenname'];
        } else {
            $screenname = lang::translate('account_profile_user') . " " . $id;
        }
        $str = html::createLink("/account_profile/view/$id", $screenname);
        return $str;
    }
    
    /**
     * retrun profile as a link
     * @param   int     $id
     * @return  string  $str html profile link
     */
    public static function getProfileEditLink ($id){
        $profile = self::getAccountProfileFromId($id);
        $str = html::createLink(
            "/account_profile/edit/$id", 
            lang::translate('account_profile_edit_link'));
        return $str;
    }

    /**
     * return full profile info
     * @param   array $user
     * @return  array $profile
     */
    public static function getProfileInfo ($user) {
        $profile = self::getAccountProfileFromId($user['id']);
        return $profile;
    }


    /**
     * gets identicon image url, creates a cached copy if identicon image
     * does not exists
     *
     * @param int   $size
     * @param string $hash key
     * @return string   the url of the identicon image
     */
    
    public static function getIdenticonImageUrl ($email, $size){
        moduleLoader::includeModel ('gravatar');
        return get_gravatar($email, $size);
    }
    
    /**
     * return account profile as a table
     * @param array $user
     * @param array $options
     * @return string
     */
    public static function createIdenticonTable ($user, $text){

        $vars = array();
        $size = config::getModuleIni('account_profile_image_size_small');
        if (!$size){
            $size = 38;
        }
        
        $vars['image_link'] = html::createHrefImage(
                "/account_profile/view/$user[id]",
                self::getIdenticonImageUrl($user['email'], $size),
                array('alt' => lang::translate('account_profile_identicon_alt')));

        
        $vars['text_link'] = self::getProfileLink($user['id']);
        
        $vars['text'] = $text;
        $str = templateView::get(
                'account_profile', 
                'account_profile_box', 
                $vars);
        return $str;
    }

    /**
     * return anon identicon table profile
     * this is used in comments, but can be used else where
     * 
     * @param array $user
     * @param array $options
     * @return string
     */
    public static function createIdenticonTableAnon ($user, $text){
        $vars = array();
        //echo $email = md5($user['email']);
        $size = config::getModuleIni('account_profile_image_size_small');
        if (!$size){
            $size = 38;
        }

        if (!empty($user['homepage'])){
            $vars['image_link'] = 
                html::createHrefImage(
                $user['homepage'],
                self::getIdenticonImageUrl($user['email'], $size),
                array('title' => lang::translate('account_profile_visit_homepage') . 
                                    ' ' . $user['homepage'] ));
        } else {
            $vars['image_link'] = 
            html::createImage(self::getIdenticonImageUrl($user['email'], $size), 
                array('alt' => lang::translate('account_profile_identicon_anonymous')));
        }

        if (!isset($user['name']) || empty($user['name'])) {
            $user['name'] = lang::translate('account_profile_anonymous');
        }
        
        if (!empty($user['homepage'])) {
            $vars['text_link'] = html::createLink($user['homepage'], $user['name']);
        } else {
            $vars['text_link'] = $user['name'];
        }
        $vars['text'] = $text;
        $str = templateView::get(
                'account_profile', 
                'account_profile_box', 
                $vars);
        return $str;
    }

    
    /**
     * method for validating a profile when setting profile.
     */
    public static function validate (){
        if (!empty($_POST)){
            $id = self::getId();
            if ($id != session::getUserId()){
                self::$errors[] = lang::translate('account_profile_account_not_yours');
            }


            if (!empty($_POST['website'])){
                if (!cosValidate::url($_POST['website'])){
                    self::$errors = lang::translate('account_profile_error_url');
                }
            }

            if (!empty($_POST['email'])){

                if (!cosValidate::validateEmailAndDomain($_POST['email'])){
                    self::$errors = lang::translate('account_profile_error_email');
                }

                $row = self::getAccountFromEmail($_POST['email']);
                if (!empty($row)){                    
                    if (session::getUserId() != $row['id']){
                        self::$errors = lang::translate('account_profile_error_email_exists');
                    }
                }
            }

            if (!empty($_POST['screenname'])) {
                $db = new db();
                $row = $db->selectOne('account_profile', 'screenname', $_POST['screenname']);

                if (isset($row['id']) && ($row['user_id'] != session::getUserId()) ) {
                    self::$errors = lang::translate('account_profile_screenname_exists');
                }
            }
        }

        if (!empty($_POST['birthday'])){
            if (!self::checkDate($_POST['birthday'])){
                self::$errors['birthday'] = lang::translate('account_profile_no_valid_date');
            }
        }
    }


    /**
     * method for sanitizing profile data
     */
    public static function sanitize (){
        $_POST = html::specialEncode($_POST);
    }

    /**
     * method for check if a date is correct or not
     * @param <type> $date
     * @return <type>
     */
    static function checkDate ($date){
        return time::checkDate($date);
    }

    /**
     * method for displaying a user profile
     * @param array $options
     */
    public static function viewProfile($id){
        $row = self::getAccountProfile($id);
        $filters = config::getModuleIni ('account_profile_filters');
        $row['description'] = moduleLoader::getFilteredContent($filters, $row['description']);
        $str = templateView::get('account_profile', 'view', $row);
        echo $str;        
    }


    public static function getAccountProfile ($id){
        $db = new db();
        $account_profile = $db->selectOne('account_profile', 'user_id', $id);
        return $account_profile;
    }

    public static function getAccount($id){
        $db = new db();
        $row = $db->selectOne('account', 'id', $id);
        return $row;
    }

    public static function getAccountFromEmail($email){
        $db = new db();
        $row = $db->selectOne('account', 'email', $email);
        if (!$row){
            return array();
        }
        return $row;
    }

    public static function getAccountProfileFromId($id){
        $db = new db();
        $row = $db->selectOne('account_profile', 'user_id', $id);
        if (!$row){
            return array();
        }
        return $row;
    }


    /**
     * function for creating a form for insert, update
     * profile entries. 
     *
     */
    public static function accountProfileForm(){

        $options = array();
        $db = new db();
        $id = self::getId();
        

        $row = $db->selectOne('account_profile', 'user_id', $id);
        if (!$row) { 
            self::updateAccountProfile ();
            $row = $db->selectOne('account_profile', 'user_id', $id);
        }
        
        $account = self::getAccount($id);

        if (empty($account['email'])){  
            $row['use_email'] = true;
        }
        templateView::includeModuleView('account_profile', 'form', $row);
    }

    public static function updateAccountProfile(){
        $id = self::getId();
        $db = new db();

        $_POST['user_id'] = $id;

        $values = db::prepareToPost();
        $row = $db->getNumRows('account_profile', array ('user_id' => $id));
        if (!$row){
            $res = $db->insert('account_profile', $values);
        } else {
            $res = $db->update('account_profile', $values, array('user_id' => $id));
        }
        return $res;
    }
}
