<?php

/**
 * File containing model for blog, all major methods.
 */

/**
 * @ignore
 */
moduleLoader::includeModule('tags');
//view::includeOverrideFunctions('blog', 'blog_views.phtml');

/**
 * Class blog
 */
class blog {

    /**
     * @var array   array for holding error codes
     */
    public static $errors = array();

    /**
     * @var string  name of the blog table.
     */
    public static $blog_table = 'blog';
    
    public static $post = array (
        'title', 'entry', 'abstract', 
        'created', 'user_id', 'updated'
    );
    
    /**
     * just a test for urldispatch
     * return string $str
     */
    public static function index () {        
        if (!session::checkAccessControl('blog_allow')){
            return;
        }
        
    }

    
    public static function loadAssets ($options = array()) {
        if (config::getModuleIni('blog_use_jquery_markedit')) {
            moduleLoader::includeTemplateCommon('jquery-markedit');
            jquery_markedit_load_assets($options);
        }
        
        template::setInlineCss(config::getModulePath('blog') .  '/assets/blog.css');        
    }


    /**
     * method for getting a filtered entry id.
     * @return int  returns filtered int
     */
    public static function getEntryId (){
        return filter_var(uri::$fragments[2], FILTER_VALIDATE_INT);
    }

    /**
     * method for getting a filtered entry id.
     * @return int  returns filtered int
     */
    public static function getEntryTitle (){
        return uri::$fragments[3];
    }

    /**
     * function for creating a form for insert, update and deleting
     * blog entries. 
     *
     * @param string    method (update, delete or insert)
     * @param int       id (if delete or update)
     * @param array     array with values if we load the form for
     *                  an update operation
     */
    public static function blogForm($method, $id = null, $values = array()){
        if ($method == 'insert'){
            include "views/entry_add.inc";
        }
        if ($method == 'update'){
            include "views/entry_edit.inc";
        }
        if ($method == 'delete') {
            include "views/entry_delete.inc";
        }
    }



    /**
     * we add a method for validating the submitted entry
     */
    public static function validate(){
        if (isset($_POST['submit'])){
            if (empty($_POST['title'])) {
                self::$errors[] = lang::translate('blog_no_title');
            }
            /*
            if (empty($_POST['entry'])) {
                self::$errors[] = lang::translate('blog_no_entry');
            }*/
        }
    }
    
    public static function sphinxHit ($id) {
        
        $row = dbQ::setSelect('blog', 'id, title, user_id, abstract, created')->
                filter('id =', $id)->
                    fetchSingle();
        if (empty($row)) return '';
        echo view::get('blog', 'index_single', $row);
        
    }


    /**
     * sanitize input on submit
     */
    public static function sanitize(){
        if (isset($_POST['submit'])){
            $_POST = html::specialEncode($_POST);
        }
    }

    /**
     * method for adding a blog entry. This is the add controller.
     * we will make a controller file where this function is called.
     *
     * @return void
     */
    public static function addController(){
        

        // we set a title for the page where this method is used
        // this is the <title> tag. in the page.
        template::setTitle(lang::translate('blog_add_title'));

        if (!session::checkAccessControl('blog_allow')){
            return;
        }

        if (isset($_POST['submit'])){
            self::sanitize();
            self::validate();

            if (empty(self::$errors)){
                $res = self::addEntry();
                if ($res){
                    session::setActionMessage(
                        lang::translate('blog_entry_created'), true
                    );
                    http::locationHeader('/blog/index');
                }
            } else {
                view_form_errors(self::$errors);
                self::blogForm('insert');
            }
        } else {
            self::blogForm('insert');
        }
    }

    public static function tagController (){

        
        $tag = tags::getTagSingle(blog::getEntryId());
        
        if (empty($tag)) {
            moduleLoader::$status['404'] = true;
            return;
        }
        
        $url = strings::utf8Slug("/blog/tags/$tag[id]", $tag['title']);
        http::permMovedHeader($url);

        $title = lang::translate('blog_tags_list_entries_title');
        $title.= MENU_SUB_SEPARATOR;
        $title.= $tag['title'];

        template::setTitle($title);
        
        $headline = lang::translate('blog_tags_list_entries_title');
        $headline.= MENU_SUB_SEPARATOR_SEC;
        $headline.= $tag['title'];
        
        headline_message($headline);

        $num_rows = tags::getAllReferenceTagNumRows('blog', self::getEntryId());
        if ($num_rows == 0){
            echo lang::translate('blog_tags_no_articles');
            return;
        }

        
        include_once "coslib/pearPager.php";
        $per_page = config::getModuleIni('blog_per_page');
        $pager = new pearPager($num_rows, $per_page);
        
        $references = tags::getAllReferenceTag ('blog', self::getEntryId(), $pager->from, $per_page);
        
        $rows = dbQ::setSelect(self::$blog_table)->
                filterIn("id IN ", $references)->
                order('created', 'DESC')->fetch();

        echo view::get('blog', 'tags', $rows);
        $pager->pearPage();

    }

    /**
     * method for listing latest entries in our blog
     *
     */
    public static function indexController (){

        
        define('PAGER_PER_PAGE' , config::getModuleIni('blog_per_page'));

        include_once "coslib/pearPager.php";
        $db = new db();
        $num_rows = $db->getNumRows(self::$blog_table);

        $pager = new pearPager($num_rows);
        
        $title = lang::translate('blog_view_all');
        $title.= MENU_SUB_SEPARATOR;
        $title.= pearPager::getPageNum();
        
        template::setTitle($title);
        
        $rows = $db->selectAll(self::$blog_table, null, null, $pager->from, PAGER_PER_PAGE, 'created');     
        
        templateView::includeModuleView('blog', 'index', $rows);
        $pager->pearPage();
    }
    
    public static function getAdminLinks ($id) {
        $str = html::createLink("/blog/edit/$id", lang::translate('blog_edit_entry'));
        $str.= MENU_SUB_SEPARATOR;
        $str.= html::createLink("/blog/delete/$id", lang::translate('blog_delete_entry'));
        return $str;
    }

    /**
     */
    public static function viewController (){

        $db = new db();
        $row = $db->selectOne(self::$blog_table, null, array ('id' => self::getEntryId()));

        if (empty($row)){
            moduleLoader::$status[404] = 1;
            return;
        }
        
        
        $url = strings::utf8Slug("/blog/view/$row[id]", $row['title']);
        http::permMovedHeader($url);

        // we set a html title for the page where this method is used
        template::setTitle($row['title'] . MENU_SUB_SEPARATOR  . lang::translate('blog_view_entry'));
        template::setMeta(array('description' => $row['abstract']));
        $filters = config::getModuleIni('blog_filters');
        $row['entry'] = moduleLoader::getFilteredContent($filters, $row['entry']);
        
        echo view::get('blog', 'view', $row);
    }

    public static function getEntry ($id = null){
        $db = new db();

        if (!$id) $id = self::getEntryId ();
        $row = $db->selectOne(self::$blog_table, null, array ('id' => $id));   
        
        if (empty($row)) {
            return $row;
        }  
        return $row;
    }

    /**
     * return link from id
     * @param type $id
     * @param type $options
     * @return type
     */
    public static function getLinkFromId ($id, $options){
        return self::getReturnLink($id, $options);
    }
    
    /**
     * gets redirect url from id
     * @param type $id
     * @return type
     */
    public static function getRedirect ($id = null){
        if (!$id) {
            $id = filter_var(uri::$fragments[3], FILTER_VALIDATE_INT);
        }
        $row = self::getEntry($id);
        $link_url = strings::utf8Slug("/blog/view/$row[id]", $row['title']);
        return $link_url;
    }

    /**
     * gets return url from id
     * @param type $id
     * @return type
     */
    public static function getReturnUrlFromId ($id){

        $row = self::getEntry($id);
        $link_url = strings::utf8Slug("/blog/view/$row[id]", $row['title']);
        return $link_url;
    }
    
    /**
     * gets a return link from id
     * @param type $id
     * @param type $options
     * @return type
     */
    public static function getReturnLink ($id = null, $options = array ()){
        
        $row = self::getEntry($id);
        if (empty($row)) {
            return lang::translate('blog_reference_removed');
        }

        $link_url = null;
        if (isset($options['edit_link'])) {
            $link_url = strings::utf8Slug("/blog/edit/$row[id]", $row['title']);        
        } else {
            $link_url = strings::utf8Slug("/blog/view/$row[id]", $row['title']); 
        }
        $link = html::createLink($link_url, $row['title'], $options);
        return $link;
    }



    /**
     * note this is almost exactly the same function as the insertController
     * only difference is that we use an id for loading the form
     *
     * @return void
     */

    public static function updateController (){
        $id = self::getEntryId();
        $row = self::getEntry($id);
        if (empty($row)) { 
            http::permMovedHeader ('/blog/index');
        }
        
        http::prg();
        template::setTitle(lang::translate('blog_update_entry'));
        if (!session::checkAccessControl('blog_allow')){
            return;
        }

        if (isset($_POST['submit'])){
            self::validate();
            self::sanitize();
            if (empty(self::$errors)){
                $res = self::updateEntry();
                if ($res){
                    session::setActionMessage(
                        lang::translate('blog_entry_updated'), true
                    );
                    http::locationHeader('/blog/index');
                } 
            } else {
                view_form_errors(self::$errors);
                self::blogForm('update', self::getEntryId());
            }
        } else {
            self::blogForm('update', self::getEntryId());
        }
    }


    /**
     * controller for deleting a entry
     * @return void
     */

    public static function deleteController (){

        // we set a title for the page where ths method is used
        template::setTitle(lang::translate('blog_delete_entry'));
        if (!session::checkAccessControl('blog_allow')){
            return;
        }

        if (isset($_POST['submit'])){
            $res = self::deleteEntry();
            if ($res){
                session::setActionMessage(
                    lang::translate('blog_entry_deleted')
                );
                http::locationHeader('/blog/index');
            }
        } else {
            self::blogForm('delete', self::getEntryId());
        }

    }

    
    /**
     * method for inserting a blog entry into database
     *
     * @return  boolean true on succes false on failure
     */
    public static function addEntry (){
        $db = new db();

        $_POST = html::specialDecode($_POST);
        db::$dbh->beginTransaction();
        // we add user id and updated fields to the _POST var
        $_POST['user_id'] = session::getUserId();
        $_POST['created'] = date('Y-m-d H:i:s');
        $_POST['updated'] = date('Y-m-d H:i:s');
        
        $post = array (
            'title', 'entry', 'abstract', 'updated', 'created'
        );
        
        $entry = db::prepareToPostArray($post);

        // prepare and execute
        // insert the blog post
        try {
            $res = $db->insert(self::$blog_table, $entry);
            $last_insert_id = db::$dbh->lastInsertId();
            
            $event_params = array (
                'action' => 'insert',
                'reference' => 'blog', 
                'parent_id' => $last_insert_id);
            
            event::triggerEvent(
                config::getModuleIni('blog_events'),
                $event_params);
            
        } catch (PDOException $e) {
            db::$dbh->rollBack();
            error_log($e->getTraceAsString());
            return false;
        }

        $res = db::$dbh->commit();
        return $res;
    }

    /**
     * method for updating an entry
     * @return boolean
     */
    public static function updateEntry (){
        $db = new db();
        db::$dbh->beginTransaction();

        $_POST = html::specialDecode($_POST);
        $_POST['updated'] = date('Y-m-d H:i:s');
        $_POST['user_id'] = session::getUserId();
        
        // don't include created array index 3
        unset(self::$post[3]); 
        
        $entry = db::prepareToPostArray(self::$post);
        try {
            $res = $db->update(self::$blog_table, $entry, self::getEntryId());
            $id = self::getEntryId();

            $event_params = array (
                'action' => 'update',
                'reference' => 'blog', 
                'parent_id' => $id);
            
            event::triggerEvent(
                config::getModuleIni('blog_events'),
                $event_params);
            
        } catch (PDOException $e) {
            db::$dbh->rollBack();
            cos_debug($e->getTraceAsString());
            return false;
        }
        return db::$dbh->commit();
    }

    /**
     * method for updating an entry
     * @return boolean
     */
    public static function deleteEntry (){
        $db = new db();
        db::$dbh->beginTransaction();
        try {
            $res = $db->delete(self::$blog_table, 'id', self::getEntryId());
            
            $event_params = array (
                'action' => 'delete',
                'reference' => 'blog', 
                'parent_id' => self::getEntryId());
            
            event::triggerEvent(
                config::getModuleIni('blog_events'),
                $event_params);

        } catch (PDOException $e) {
            db::$dbh->rollBack();
            error_log($e->getTraceAsString());
            return false;
        }
        return db::$dbh->commit();
    }

    /**
     * method for getting user info
     * this is only used in the templates as we will see in the next session
     * 
     * @param   int     $id
     * @return  array   account row
     */
    public static function getUserInfo ($id){
        $db = new db();       
        $row = $db->selectOne('account', 'id', $id);
        return $row;
    }

    


    /**
     * method for getting latest articles
     *
     * @param   int from
     * @param   int limit
     * @return  array   with latest article rows
     */
    public static function getRowsRSS($from = 0, $limit = 20){
            
            $rows = dbQ::setSelect(self::$blog_table)
                    ->order('created', 'DESC')
                    ->limit($from, $limit)         
                    ->fetch();

            foreach ($rows as $key => $val){
                $rows[$key]['url'] = self::getReturnUrlFromId($val['id']);
            }
            
            return $rows;
    }
}
