### About

A comment module for coscms. Can be used with other modules. 
See 'blog', 'questions' or 'content' modules in order to see how it is done.

### Configuration  

    ; comment view filter
    comment_view_filter = "markdown"
    ; allow anon posts
    comment_allow_anon_post = 1
    ; always verify anon post
    comment_anon_post_verify = 1
    ; verify signed up users posts
    comment_user_post_verify = 0
    ; use aksimet
    comment_use_akismet = 0
    ; akismet key
    comment_akismet_key = "aksimet_key"
    ; use jquery markdown editor
    comment_use_jquery_markdown = 1
    ; who can edit comments
    comment_allow_edit_comment = "admin"
    ; disable title in comment
    comment_disable_title = 1
    ; comment events
    comment_events[0] = "vote::events"
    comment_events[1] = "flood::events"
    ; sort comments by
    comment_sort_by = "vote"
    ; send user a notification when a comment has been accepted
    comment_send_notification = 1

### Sub module system

This is an example of a generic way of adding sub modules to a main module. 
You will need the following 2 methods in a module in order to
make a module that 'can' function as a sub module. Even if it can function
as a sub module it can still function as a main module.

In order to use a sub module we use an 'id' and a 'reference'. The 'id' is the
parent 'id', e.g. a blog entry, and the reference shold be the parent module's
name, e.g. 'blog' or 'questions'. 
 
    /**
     * method for using comment module as a subModule
     * @param array $options e.g. array ('parent_id' =1, 
     *                                   'reference' => 'questions',
     *                                   'return_url => '/questions/1/new-question');   
     *                                   
     * @return string $str comments connected to reference and parent_id
     */
    public static function subModulePostContent($options) {
        // return all comments connected to e.g. blog
        return $post_contet;
    }

    /**
     * method for adding comments as a sub module.
     *
     * @param array $options e.g. array ('parent_id' =1, 
     *                                   'reference' => 'questions',
     *                                   'return_url => '/questions/1/new-question'); 
     * @return string $str string containing note about have many comments is
     *                     connected to reference and parent_id
     */
    public static function subModulePreContent ($options = array()){
        // return what comes before main module's content
        // with the comments modules, it is a count of comments
        return $pre_content
    }

Another method is available. This is subModuleInlineContent
This is not used in the comments module, but is available - this method collects
content to be parsed jut before the content. 

### comments as a sub module

The next code shows how comments are enabled on the blog module, As you can see
you can make connect any other sub module do the same. 

    // sub modules to use are specified in a ini setting where comment
    // is one of the enabled view_sub_modules
    // blog_view_sub_modules[] = 'comment'

    // get all sub modules
    $view_modules = config::getModuleIni('blog_view_sub_modules');

    // include all sub modules
    moduleLoader::includeModules($view_modules);

    // compose options to give to sub modules
    $return_url = blog::getReturnUrlFromId($vars['id']);
    $options = array (
        'parent_id' => $vars['id'], // the blog id
        'reference' => 'blog', // the reference - module name 'blog'
        'return_url' => $return_url // return url on a action
    );

    // get pre content of all modules
    $pre = moduleLoader::subModuleGetPreContent($view_modules, $options);
    $ary = array_merge($ary, $pre);

    // echo pre content
    echo implode("<hr />\n", $ary);
    echo "<div id=\"toc\"></div>\n";

    // get inline content of all sub modules
    $inline = moduleLoader::subModuleGetInlineContent($view_modules, $options);

    // echo all inline content
    echo implode("<hr />\n", $inline);

    // echo the blog entry
    echo "<div id=\"blog_content\">\n";
    echo $vars['entry'];
    echo "</div>\n";
    echo "<hr />\n";

    // get all post content
    $ary = moduleLoader::subModuleGetPostContent($view_modules, $options);

    // echo all post content
    echo implode("<hr />\n", $ary);
    echo "</article>\n";

### Methods needed in the main module

In order for the sub module to know where to return to after a post, and also
displaying a link to return to from the post we need a couple of functions
in our main module in order to implement this.

You need a redirect URL on a successful post. 

Therefor you implement the following method in your main modules main class. 
You don't always needs these method. But when you have them you can connect
a few existing modules, e.g. files, video, image, and comments. 

    public static function getLinkFromId ($id, $options){
        // get main entry from the id given - e.g. a blog id
        // and return <a></a> link - e.g. <a href="/blog/view/12">My entry</a>
        return $str;
    }

    public static function getRedirect ($id = null){
        return $str; // redirect - used e.g. on success when posting a comment. 
    }

