<?php

/**
 * model file for comment module
 *
 * @package comment
 */

/**
 * @ignore
 */
include_once "coslib/captcha.php";
comment::loadAssets();

/**
 * class comment
 *
 * @package content
 */
class comment {

    /**
     *
     * @var array holding errors on submission
     */
    public $errors = array();

    /**
     *
     * @var object holding URI info
     */
    public $uri;

    /**
     *
     * @var int current article id
     */
    public $id;

    /**
     * @var array   options to constructor
     */
    public $options;

    /**
     * sets the id for the page (used for update and delete)
     * @param array $options
     */
    function __construct($options = null){
        $this->id = self::getCommentId();
        if (isset($options)){
            $this->options = $options;
        }
    }
    
    /**
     * method for loading all assets
     */
    public static function loadAssets () {
        
        if (config::getModuleIni('comment_use_jquery_markdown')) {
            moduleLoader::includeTemplateCommon('jquery-markedit');
            jquery_markedit_load_assets();
        }
        template::setInlineCss(config::getModulePath('comment') . "/assets/comment.css");
    }

    /**
     * method for using comment module as a subModule
     * @param array $options e.g. array ('parent_id' =1, 
     *                                   'reference' => 'questions',
     *                                   'return_url => '/questions/1/new-question');   
     *                                   
     * @return string $str comments conneted to reference and parent_id 
     */
    public static function subModulePostContent($options) {
        ob_start();
        viewComment::viewComments($options['parent_id'], $options);
        $comments = ob_get_contents();
        ob_end_clean();
        return $comments;
    }

    /**
     * method for adding comments as a sub module before the reference modules
     * content. 
     * @param array $options e.g. array ('parent_id' =1, 
     *                                   'reference' => 'questions',
     *                                   'return_url => '/questions/1/new-question'); 
     * @return string $str string containing note about have many comments is
     *                     connected to reference and parent_id
     */
    public static function subModulePreContent ($options = array()){
        $num_rows = self::getNumVerifiedComments($options['parent_id'], $options['reference'] );
        if ($num_rows == 1) {
            $message = lang::translate('comment_view_one_comment');
        } else {
            $message = lang::translate('comment_view_comments');
        }
        $message = $num_rows . ' ' . $message;
        return html::createLink('#comments', $message);
    }

    /**
     * return number of rows connected to a parent id
     * @param   int    $parent_id
     * @param   string $reference
     * @return  int    $num number of verfied comment connect to parent and 
     *                      reference
     */
    public static function getNumVerifiedComments ($parent_id, $reference){
        $db = new db();
        $num_comments = $db->getNumRows('comment', 
                array (
                    'parent_id' => $parent_id,
                    'admin_verified' => 1,
                    'reference' => $reference));
        return $num_comments;
    }

    /**
     * gets a comments id from URL. 
     * @return  int  comment id
     */
    public static function getCommentId (){
        $uri = uri::getInstance();
        $id = (int)$uri->fragment(3);
        return $id;

    }
    
    /**
     * method for validating user input when posting a comment
     * Validate user input
     * return void
     */
    public function validate(){
        if (empty($_POST['submit'])){
            return;
        }

        $disable_title = config::getModuleIni('comment_disable_title');
        if (!$disable_title) {
            if (strlen($_POST['title']) < 3){
                $this->errors['title'] = lang::translate('comment_error_title');
            }
        }

        if (config::getModuleIni('comment_allow_anon_post') && !session::isUser()){
            $options = array(
                'check_domain' => 'true',
                'fullTLDValidation' => 'true',
                'use_rfc822' => 'true',
                'VALIDATE_GTLD_EMAILS' => 'true',
                'VALIDATE_CCTLD_EMAILS' => 'true',
                'VALIDATE_ITLD_EMAILS' => 'true',
            );
            
            // validate email as good as we can
            if (!cosValidate::validateEmailAndDomain($_POST['email'], $options)){
                $this->errors['email'] = lang::translate('comment_error_email');
            }

            $homepage = trim($_POST['homepage']);
            if (!empty($homepage)){
                if (!cosValidate::url($_POST['homepage'])){
                    $this->errors['homepage'] = lang::translate('comment_error_homepage');
                }
            }
        }

        if (strlen($_POST['comment']) < 3){
            $this->errors['comment'] = lang::translate('comment_error_content');
            //return;
        }

        // user allowed to escape captcha
        if (!session::isUser() && !captcha::checkCaptcha($_POST['captcha'])) {
            $this->errors['captcha'] = lang::translate('comment_captcha_error');
        }
    }

    /**
     * method used for sanitizing comment submission
     * just encoding special chars
     */
    public function sanitize(){
        $_POST = html::specialEncode($_POST);
    }

    /**
     * calculate if comment is verfied
     * @return  int  0 or 1
     */

    /**
     * returns if user is verified or not based on sessions
     * @return int $ret
     */
    public static function getIsVerified (){
        $verified = 1;

        // check if auto verfiy anon comments
        if (!session::isUser()){
            if (config::getModuleIni('comment_anon_post_verify')){
                $verified = 0;
            } else {
                $verified = 1;
            }
        }

        // check if we auto verify user comments
        if (session::isUser()) {
            if (config::getModuleIni('comment_user_post_verify')){
                $verified = 0;
            } else {
                $verified = 1;
            }
        }

        // Admins can always create verified comment
        if (session::isAdmin()){
            $verified = 1;
        }
        return $verified;
    }

    /**
     * method for creating a comment in comment in comment table
     * @return boolean $res
     */
    public function createComment (){
        $db = new db();
        $verified = self::getIsVerified();
        $_POST['admin_verified'] = $verified;
        
        // check if we are inline another module, and we are posting a comment
        // to another comment
        if ($this->options['inline_parent_id']) {
            $_POST['parent_id'] = $this->options['inline_parent_id'];
            $_POST['reference'] = 'comment';
        } else {
            $_POST['parent_id'] = $this->options['parent_id'];
            $_POST['reference'] = $this->options['reference'];
        }
        
        // check aksimet if config tells us
        if (config::getModuleIni('comment_use_akismet')){
            $res = self::akismetCommentValid($_POST['comment']);
            if (!$res){
                $_POST['admin_verified'] = 0;
            }
        }

        $_POST['user_id'] = session::getUserId();

        // decode HTML when we put it into DB.
        $_POST = html::specialDecode($_POST);
        $values = db::prepareToPost();
        
        $db->begin();
        $res = $db->insert('comment', $values);
        
        $args = array (
            'reference' => 'comment',
            'action' => 'comment_create'
        );
        
        event::getTriggerEvent(
            get_module_ini('comment_events'), $args
        );
        
        if ($res) {           
            $db->commit();
            return true;
        } else {
            $db->rollback();
            return false;
        }
        
        //return $res;
    }

    /**
     * method for updating a comment
     * @return boolean $res
     */
    public function updateComment (){
        $db = new db();
        // we decode in order to get it correct when displaying as markdown.
        $_POST= html::specialDecode($_POST);

        $values = db::prepareToPost();
        $res = $db->update('comment', $values, $this->id);
        return $res;
    }

    /**
     * method for delting a comment
     *
     * @param   int     $id id of comment
     * @return  boolean $res true on success and false on failure
     *
     */
    public function deleteComment($id){
        $db = new db();
        $res = $db->delete('comment', 'id', $id);
        return $res;
    }
    
    public static function deleteReferenceId($parent, $reference) {

        return dbQ::setDelete('comment')->filter('reference =', $reference)
                ->condition('AND')
                ->filter('parent_id =', $parent)
                ->exec();
               
    }

    /**
     * method for getting comments that are verfified
     *
     * @param int $parent_id
     * @param int $reference the reference
     * @return array $ary assoc row of all comments connected to reference and
     *                    parent_id
     */
    
    public static function getComments ($parent_id, $reference){
        
        $sort_by = config::getModuleIni('comment_sort_by');
        if (!$sort_by) {
            $sort_by = 'created';
        }

        $rows = dbQ::setSelect('comment')->
            filter('parent_id =', $parent_id)->
            condition('AND')->
            filter('reference =', $reference)->
            order($sort_by, 'DESC')->
            fetch();
        return $rows;
    }
    
    /**
     *
     * @param string $reference
     * @param int $from
     * @param int $limit
     * @return array $comments 
     */
    public function getCommentsToReference($reference, $from = 0, $limit = 10){
        $db = new db();
        $search = array('reference' => $reference);
        $rows = $db->selectAll('comment', null, $search, null, null, 'created', null);
        return $rows; 
    }

    /**
     *
     * @param int $from
     * @param int $limit
     * @param array $search
     * @return array $comments 
     */
    public function getAllComments($from = 0, $limit = 10, $search = null){
        $db = new db();
        $rows = $db->selectAll('comment', null, $search, $from, $limit, 'created', null);
        return $rows; 
    }
    
    /**
     * get filter for comments from session
     * @return string $filter
     */
    public static function getCommentFilter () {
        if (!isset($_SESSION['comment_filter'])) {
            $_SESSION['comment_filter'] = 1;
        }
        
        if (isset($_GET['comment_filter'])) {
            $_SESSION['comment_filter'] = $_GET['comment_filter'];
        }
        
        return $_SESSION['comment_filter'];
    }

    /**
     * get number of comments
     * @return int $num 
     */
    public function getNumComments (){
        $db = new db();
        $filter = comment::getCommentFilter();
        if ($filter == 1) {
            return $num_rows = $db->getNumRows('comment', array('admin_verified' => '0'));
        }
        
        $num_rows = $db->getNumRows('comment');
        return $num_rows;
    }

    /**
     * get info about user 
     * @param   int     user id
     * @return  array   row with userinfo
     */
    public static function getUserInfo($id){
        $db = new db();
        $search = array('id' => $id);
        $row = $db->selectOne('account', null, $search);
        return $row;
    }

    /** 
     * method for displaying a form for creating a comment
     */
    public function viewCommentForm(){
        http::prg();

        // create comment object and print comment form
        if (!config::getModuleIni('comment_allow_anon_post')){
            session::loginThenRedirect(lang::translate('comment_login_in_order_to_post'));
        }
        
        $this->sanitize();
        if (isset($_POST['submit'])){
            $this->validate();
            if (empty($this->errors)){          
                $res = $this->createComment();
                if ($res){
                    if (self::getIsVerified()){
                        $message = lang::translate('comment_has_been_verfied');
                    } else {
                        $message = lang::translate('comment_needs_to_be_verified');
                    }
                    session::setActionMessage($message, true);
                    http::locationHeader($this->options['redirect']);
                }
            } else {
                view_form_errors($this->errors);
            }
        }
        viewComment::viewCommentForm('insert');
    }

    /**
     * method displaying an edit form for a comment
     */
    public function viewEditCommentForm(){
        if (isset($_POST['submit'])){
            $this->sanitize();
            $this->validate();
            if (empty($this->errors)){
                $res = $this->updateComment();
                if ($res){
                    session::setActionMessage(
                        lang::translate('comment_updated'));
                    http::locationHeader($this->options['redirect']);
                }
            } else {
                view_form_errors($this->errors);
            }
        }
        viewComment::viewCommentForm('update', $this->id);
    }

    /**
     * method displaying an edit form for a comment
     */
    public function viewAcceptCommentForm(){
        if (isset($_POST['submit'])){
            $this->sanitize();
            $this->validate();
            if (empty($this->errors)){
                $res = $this->updateComment();
                if ($res){
                    session::setActionMessage(
                    lang::translate('comment_updated'));
                    http::locationHeader($this->options['redirect']);
                    
                }
            } else {
                view_form_errors($this->errors);
            }
        }
        viewComment::viewCommentForm('update', $this->id);
    }
    
    /**
     * 
     * @param type $value
     * @return type 
     */
    public static function getCommentUrlFromId (&$value) {
        moduleLoader::includeModule ($value['reference']);
        
        // get reference to parent
        if ($value['reference'] == 'comment') {
            $value = self::getComment($value['parent_id']);
        }
        
        $class = moduleLoader::modulePathToClassName($value['reference']);  
        $reference = $class::getRedirect($value['parent_id']);
        //} else {
        //$reference = $class::getRedirect($value['parent_id']);
        //}
        return $reference;
    }
    
    /**
     * Method for getting a comments reference link
     * @param array $row
     * @return string $link
     */
    public static function getReferenceLink (&$value) {
        moduleLoader::includeModule ($value['reference']);
        $class = moduleLoader::modulePathToClassName($value['reference']);    
        $anchor = "#comment$value[id]";
        $options['anchor_part'] = $anchor;
        $reference = $class::getLinkFromId($value['parent_id'], $options);
        return $reference;
    }
    
    /**
     * method for displaying an form for deleting a comment
     */
    public function viewDeleteCommentForm(){
        if (isset($_POST['submit'])){
            if (empty($this->errors)){
                $res = $this->deleteComment($this->id);
                if ($res){
                    session::setActionMessage(
                        lang::translate('comment_deleted'));
                    http::locationHeader($this->options['redirect']);
                }
            } else {
                view_form_errors($this->errors);
            }
        }
        viewComment::viewCommentForm('delete', $this->id);
    }

    /**
     * accept a comment
     * @param int $id
     * @return boolean $res true on success and false on failure 
     */
    public function acceptComment($id){
        $db = new db();
        $values = array ('id' => $id);
        $row = $db->selectOne('comment', null, $values);

        if ($row['admin_verified'] == 1){
            $values = array('admin_verified' => 0);
            $message = lang::translate('comment_undo_accept_done');
        } else {
            $message = lang::translate('comment_accept_done');
            $values = array('admin_verified' => 1);
        }
        
        if ($row['notification_sent'] != 1 && config::getModuleIni('comment_send_notification')) {
            $subject = lang::translate('comment_notification_subject');
             
            
            $website = "http://" . config::getMainIni('server_name');
            
            $reference_url = $website . self::getCommentUrlFromId($row);
            $ary = array ($reference_url, $website);
            
            $mail_message = lang::translate('comment_notification_message', $ary);
            //die;
            mail_utf8($row['email'], $subject, $mail_message);
            
            $values['notification_sent'] = 1;
        }

        $res = $db->update('comment', $values, $id);
        session::setActionMessage($message);
        http::locationHeader($this->options['redirect']);
        return $res;
    }

    /**
     * get a comment from the comments id
     * @param   int     $id
     * @return  array   $row
     */
    public static function getComment ($id = false){
        $db = new db();
        if (!$id) $id = self::getCommentId ();
        $row = $db->selectOne('comment', null, array('id' => $id));
        return $row;
    }
    
    /**
     * validate with aksimet
     * @param string $comment
     * @return boolean $res
     */
    public static function akismetCommentValid($content){
        require_once 'Services/Akismet2.php';
        require_once 'Services/Akismet2/Comment.php';

        $comment = new Services_Akismet2_Comment(array(
            'comment_content'      => $content
        ));

        try {
            $apiKey  = config::getModuleIni('comment_akismet_key');
            
            if (config::getModuleIni('comment_server_name')){
                $my_host = config::getModuleIni('comment_server_name');
            } else {
                $my_host = "http:://" . $_SERVER['SERVER_NAME'];

            }

            $akismet = new Services_Akismet2($my_host, $apiKey);

            if ($akismet->isSpam($comment)) {
                return false;
            } else {
                return true;
            }
        } catch (Services_Akismet2_InvalidApiKeyException $keyException) {
            echo 'Invalid API key!';
            echo $keyException->getMessage();
        } catch (Services_Akismet2_HttpException $httpException) {
            echo 'Error communicating with Akismet API server: ' .
                $httpException->getMessage();
        } catch (Services_Akismet2_InvalidCommentException $commentException) {
            echo 'Specified comment is missing one or more required fields.' .
                $commentException->getMessage();
        }
    }
    
    /**
     * return a comment url from id
     * @param int $id
     * @return string $url 
     */
    public static function getReturnUrlFromId ($id){
        $link_url = "/comment/admin/index/$id";
        return $link_url;
    }
    
    /**
     * Method for getting a link from a id. 
     * standard method when using sub modules. 
     * 
     * @param int $id
     * @return string $link 
     */
    public static function getLinkFromId ($id){
        $link_url = "/comment/admin/index/$id";
        $link = html::createLink($link_url, $id);
        return $link;
    }
    
    /**
     * method used when we have successfully done something with our 
     * sub module, then we need where to redirect. 
     * @param int $id id of the discussion
     * @return string $url the url where we redirect to. 
     */
    
    public static function getRedirect ($id){
        
        $url = "/comment/admin/index/$id";
        return $url;
    }
}
