<?php

/**
 * view file for module comment
 * holds view methods
 *
 * @package    comment
 */

class viewComment {

    /**
     * comment form
     *
     * @param   string  $method
     * @param   int     $id
     */
    public static function viewCommentForm ($method, $id = null){        
        if ($method == 'insert' || $method == 'update'){
            include _COS_PATH . "/modules/comment/views/form.inc";
        }

        if ($method == 'delete'){
            include _COS_PATH . "/modules/comment/views/delete.inc";
        }
    }
    
    /**
     * outputs comments
     * @param   int     $parent_id
     * @param   array   $options options 
     */
    public static function viewComments($parent_id, $options){
        $comments = comment::getComments($parent_id, $options['reference']);
        
        echo '<div id = "comment">' . "\n";
        echo '<a name="comments">' . '</a>' . "\n";

        headline_message(lang::translate('comments'));
        
        $url_base = '/comment/add';
        $url = moduleLoader::buildReferenceURL($url_base, $options);
        $add_link = html::createLink($url, lang::translate('comment_add_link'));

        headline_message($add_link);
        self::viewCommentsListed($comments, $options);
        echo "</div>\n";
    }

    /**
     * methods for viewing comments as admin
     * @param   array   $comments db rows with comments
     */
    public static function viewCommentsAdmin($comments, $return_url = null){
        
        echo '<div id = "comment">' . "\n";
        echo '<a name="comments">' . '</a>' . "\n";

        headline_message(lang::translate('comments'));
        self::viewCommentsAdminListed($comments, $return_url, true);

        echo "</div>\n";
    }
    
    /**
     * outputs admin interface
     */
    public static function viewAdmin () {
        $options = array();
        $comment = new comment($options);

        self::viewAdminDropdown();
        $num_rows = $comment->getNumComments();
        include_once "pearPager.php";

        $pager = new pearPager($num_rows);
        
        $filter = comment::getCommentFilter();
        if ($filter == 1) {
            $search = array ('admin_verified' => '0');
        } else {
            $search = null;
        }
        
        $comments = $comment->getAllComments($pager->from, PAGER_PER_PAGE, $search);
        viewComment::viewCommentsAdmin($comments);
        $pager->pearPage();
    }
        
    /**
     * displays a drop down to select verified or non 
     * verified comments
     */
    public static function viewAdminDropdown () {

        $filter = comment::getCommentFilter();    
        
        $extras = array();
        $extras[0] = array('id' => '0', 'title' => lang::translate('comment_admin_show_all'));
        
        $rows = array (
            array(
                'title' => lang::translate('comment_admin_show_not_admin_verfied'),
                'id' => 1
            ),
            
            array(
                'title' => lang::translate('comment_admin_show all'),
                'id' => 2
            ),
               
        );
       
        
        $extras = array('onChange' => "this.form.submit()");
        
        html::$autoLoadTrigger = 'submit';
        html::init($_GET);
        html::formStart('comment_dropdown', 'get');
        html::legend(lang::translate('comment_filter_comments'));
        html::select('comment_filter', $rows, 'title', 'id', $filter, $extras);
        html::formEnd();
        echo html::getStr();

    }
    
    /**
     * outputs headline
     * @param array $row 
     */
    public static function headline (&$value){
        $str = '';
        $str.="$value[title]<a name=\"comment$value[id]\"></a>";
        headline_message($str);

        if (session::isAdmin()){
            $reference = "";            
            if ($value['reference'] == 'comment') {
                $reference.= lang::translate('comment_reply');
                $reference.= MENU_SUB_SEPARATOR_SEC;
                $reference.= self::getParentReferenceLink($value);
                echo $reference;
            } else {
                $reference.= lang::translate('comment_to_url');
                $reference.= MENU_SUB_SEPARATOR_SEC;
                $reference.= self::getReferenceLink($value);
                echo $reference;
            }            
        }
    }
    
    /**
     * Method for getting a comments reference link
     * @param array $row
     * @return string $link
     */
    public static function getReferenceLink (&$value) {
        return comment::getReferenceLink($value);
    }
    
    /**
     * method for getting a parent's reference link 
     * @param array $value the comment row
     * @return string $link the parent reference link. 
     */
    public static function getParentReferenceLink (&$value) {
        $db = new db();
        $row = $db->selectOne('comment', 'id', $value['parent_id']);
        
        moduleLoader::includeModule ($row['reference']);
        $class = moduleLoader::modulePathToClassName($row['reference']);    
        
        $anchor = "#comment$value[id]";
        $options['anchor_part'] = $anchor;
        
        $reference = $class::getLinkFromId($row['parent_id'], $options);
        return $reference;
    }

    /**
     * outputs headline 
     * @param array $row 
     */
    public static function viewHeadline (&$value){
        $str = '';
        $str.="<a name=\"comment$value[id]\"></a>";
        
        $disable_title = config::getModuleIni('comment_disable_title');
        if (!$disable_title) {
            $str.= $value['title'];
            
        }
        headline_message($str);
    }

    /**
     * outputs comments
     * 
     * @param   array   $rows rows of comments
     * @param   array   $options options 
     */
    public static function viewCommentsListed(&$comments, $options){
        
        foreach ($comments as $value){
            if ($value['admin_verified'] == 0) continue;
            
            echo "<article class=\"comment\">\n";
            echo "<div class = \"comment_item\">\n";

            // headline
            self::viewHeadline($value);
            
            // comment
            echo "<p>\n";

            $date_str = time::getDateString($value['created']);
            $user = comment::getUserInfo($value['user_id']);

            if (!$user){
                // anonymous comment
                $user = array ();
                $user['id'] = 0;
                $user['name'] = $value['name'];
                $user['homepage'] = $value['homepage'];
                $user['email'] = $value['email'];
            }

            echo user::getProfile($user, $date_str);
            
            // trigger events set in module_ini comment_events
            $args = array (
                'reference' => 'comment', 
                'action' => 'view',
                'parent_id' => $value['id']);
            
            echo event::triggerEvent(
                    config::getModuleIni('comment_events'), 
                    $args);
            
            $filters = config::getModuleIni('comment_view_filter');
            $value['comment'] = moduleLoader::getFilteredContent($filters, $value['comment']);

            echo $value['comment'];            
            // get sub comments
            self::viewSubComments($value, $options);
            echo "</p>";
            echo "</div>\n";
            echo "</article>\n";
        }
    }

    /**
     * view replies to a comment
     * @param array $row comment row
     * @param array $options 
     */
    public static function viewSubComments ($value, $options) {
        $comments = comment::getComments($value['id'], 'comment');
               
        $url_base = '/comment/add';       
        $options['id'] = $value['id'];
        $url = moduleLoader::buildReferenceURL($url_base, $options);        
        $add_link = html::createLink($url, lang::translate('comment_reply'));
        
        echo $add_link;
        //echo count($comments); echo "Replies"; echo $add_link;
        
        foreach ($comments as $value){
            if ($value['admin_verified'] == 0) continue;
            
            echo "<div class = \"comment_sub\">\n";
            self::viewHeadline($value);
            echo "<p>\n";

            $date_str = time::getDateString($value['created']);
            $user = comment::getUserInfo($value['user_id']);

            if (!$user){
                // anonymous comment
                $user = array ();
                $user['id'] = 0;
                $user['name'] = $value['name'];
                $user['homepage'] = $value['homepage'];
                $user['email'] = $value['email'];
            }

            echo user::getProfile($user, $date_str);
            
            // trigger events set in module_ini comment_events
            $args = array (
                'reference' => 'comment', 
                'parent_id' => $value['id'],
                'action' => 'view');
            
            echo event::triggerEvent(
                    config::getModuleIni('comment_events'), 
                    $args
            );
            
            $filters = config::getModuleIni('comment_view_filter');
            $value['comment'] = moduleLoader::getFilteredContent($filters, $value['comment']);

            echo $value['comment']; 
            echo "</p>";
            echo "</div>\n";
        }
    }

    /**
     * view admin interface comments
     * @param array $comments
     * @param string $return_url
     * @param string $admin_mode 
     */
    public static function viewCommentsAdminListed(&$comments, $return_url = null, $admin_mode = null){

        foreach ($comments as $value){
            if (!session::isAdmin() && $value['admin_verified'] == 0) { 
                continue;
            }

            echo "<div id = \"comment_item\">\n";
            self::headline($value);
            echo "<p>\n";

            $date_str = time::getDateString($value['created']);
            $user = comment::getUserInfo($value['user_id']);

            // anon comment
            if (!$user){
                $user = array ();
                $user['id'] = 0;
                $user['name'] = $value['name'];
                $user['homepage'] = $value['homepage'];
                $user['email'] = $value['email'];
            }

            echo user::getProfile($user, $date_str);
            $filters = config::getModuleIni('comment_view_filter');
            $value['comment'] = moduleLoader::getFilteredContent($filters, $value['comment']);

            echo $value['comment'];
            echo self::getAdminOptions ($value);
            echo "<hr />\n";
            echo "</p>";
            echo "</div>\n";
        }
    }

    /**
     * get admin options
     * @param array $row
     * @return string 
     */
    public static function getAdminOptions (&$value) {
        $get_params = "?return_url=$_SERVER[REQUEST_URI]";
        $options = array ();
        $options[] = html::createLink("/comment/admin/edit/$value[id]$get_params", lang::translate('comment_edit'));
        $options[] = html::createLink("/comment/admin/delete/$value[id]$get_params", lang::translate('comment_delete'));
        if (!$value['admin_verified']){
            $options[] = html::createLink("/comment/admin/accept/$value[id]$get_params", lang::translate('comment_accept'));
        } else {
            $options[] = html::createLink("/comment/admin/accept/$value[id]$get_params", lang::translate('comment_undo_accept'));
        }

        $i = count($options) -1; $str ='';
        foreach ($options as $key => $val){
            if ($i != 0) {
                $str.=$options[$i] . MENU_SUB_SEPARATOR;
            } else {
                $str.=$options[$i];
            }
            $i--;
        }
        return $str;
    }
}
