<?php

class configdb {

    public static $dbTable = 'config';
    
    /**
     * method which gets all config string which is set
     * @return type 
     */
    public static function getAll () {
        return db::init()->selectAll(self::$dbTable);
    }

    /**
     *
     * @param string $id the config id to be set 
     * @param string $data the data to set the id with
     * @param string $type 'module' or 'main'
     * @return boolean result from insert operation true or false
     */
    public static function set ($id, $data, $type = 'module') {
        $search = array ('id' => $id);
        db::init()->delete(self::$dbTable, null, $search);
        $values = array ('id' => $id, 'data' => serialize($data), 'type' => $type);
        return db::init()->insert(self::$dbTable, $values);
    }

    /**
     * delete a config setting
     * @param string $id the name of the config string to delete
     * @return boolean result from db::delete operation true or false 
     *//*
    public static function delete ($id) {
        $db = new db();
        $search = array ('id' => $id);
        return $db->delete(self::$dbTable, null, $search);
    }*/
    
    /**
     * get config values from file
     * @param string $module
     * @return array $db_config 
     */
    public static function getConfigValues ($module) {
        static $cache = array();
        
        if (isset($cache[$module])) {
            return $cache[$module];
        }
        $config_file = _COS_PATH . "/modules/$module/configdb.inc";
        include $config_file; 
        $cache[$module] = $db_config;        
        return $db_config;
    }
    
    /**
     * generate form from db_config file and print form
     * @param type $module 
     */
    public static function generateConfigForm ($module, $options) {
        
        $db_config = self::getConfigValues($module);
        $values = self::getFormInitValues($module);        
        $values = html::specialEncode($values);

        html::$autoLoadTrigger = 'configdb_submit';
        html::init($values);  
        html::formStart();
        
        $human_legend = $module . "_config_description";
        $human_legend_translated = lang::translate($human_legend);
        // use legend as title
        template::setTitle($human_legend_translated);
        html::legend($human_legend_translated);
           
        foreach($db_config as $key => $val) {     
            if ($val['type'] == 'textarea') { 
                html::label($val['name'], $val['description']);   
                if (!isset($val['extra'])) $val['extra'] = array();
                html::textarea($val['name'], null, $val['extra']);              
            }
            
            if ($val['type'] == 'textareaSmall') { 
                html::label($val['name'], $val['description']);   
                if (!isset($val['extra'])) $val['extra'] = array();
                html::textareaSmall($val['name'], null, $val['extra']);              
            }
            
            if ($val['type'] == 'text') {  
                html::label($val['name'], $val['description']);          
                if (!isset($val['extra'])) $val['extra'] = array();
                html::text($val['name'], null, $val['extra']);
            }
            
        }
        html::submit('configdb_submit', 'submit');
        html::formEnd(); 
        echo html::getStr();   
    }
    
    /**
     * method for getting values to init form with
     * @param type $module
     * @return type 
     */
    public static function getFormInitValues ($module) {
        $db_config = self::getConfigValues($module); 
        
        $ary = array();
        foreach($db_config as $val) {
            $setting = config::getModuleIni($val['name']);
            if ($setting) {
                $val['value'] = $setting;
            }
                       
            $ary[$val['name']] = $val['value'];
            
        }
        return $ary;       
    }
    
    /**
     * main controller method for displaying config form and doing
     * updates and insert
     * @param string $module 
     */
    public static function displayConfig ($module) {
        if (!session::checkAccessControl('configdb_allow_edit')) {
            return;
        }
        
        $config = self::getConfigValues($module, $options = null);
              
        if (isset($_POST['configdb_submit'])) {
            $values = db::prepareToPost();
            $values = html::specialDecode($values);    
                        
            db::$dbh->beginTransaction();
            foreach ($values as $key => $val) {
                self::set($key, $val);
            }           
            db::$dbh->commit();
            session::setActionMessage(lang::translate('config_settings_updated'));
            $redirect = "/$module/config/index";
            http::locationHeader($redirect);
            exit;
            
        }
        self::generateConfigForm($module, $options);
    }
    
    public function runLevel($level){

        if ($level == 2){
            $rows = self::getAll();
            foreach ($rows as $val) {
                $val['data'] = unserialize($val['data']);
                if ($val['type'] == 'module') {
                    if (!empty($val['filters'])) {
                        $val['filters'] = unserialize($val['filters']);
                        $val['data'] = moduleLoader::getFilteredContent($val['filters'], $val['data']);
                    }

                    config::$vars['coscms_main']['module'][$val['id']] = $val['data'];
                }
                
                if ($val['type'] == 'main') {
                    config::$vars['coscms_main'][$val['id']] = $val['data'];
                }
            }
        }
    }
}
