<?php

include_once "coslib/cache.php";
include_once "coslib/menu.php";
/**
 * model file for article
 *
 * @package content
 */

/**
 * article class
 *
 * @package content
 */
class contentArticle  {

    /**
     * var holding errors
     * @var array $errors holding errors on submission
     */
    public static $errors = array();

    /**
     * var holding an URI instance
     * @var object $uri holding an URI instance
     */
    public static $uri;

    /**
     *
     * @var int $id article id
     */
    public static $id;

    /**
     *
     * var holding an article before being filtered
     * @var array  $articleRaw without any filtering.
     */
    public static $articleRaw = array();

    /**
     * var holding reference name for content article
     * @var string  name of reference for any sub modules.
     */
    public static $reference = 'content/article';
    
    /**
     * var holding default post fields
     * @var array $post default content article fields
     */
    public static $post = array(
        'abstract', 'disable_headline', 'user_id', 'published', 'updated',
        'created', 'title', 'article', 'is_frontpage', 'is_menu');
                   

    /**
     * article constructor
     * checks input data if we are updating or inserting
     * sets the id for the page, used for update and delete
     * 
     * @param   boolean     we sanitize _POST if true and we do not if false
     *
     */
    function __construct(){
        self::$uri = uri::getInstance();
        self::$id = (int)self::$uri->fragment(3);      
    }

    /**
     * method for loading markedit assets
     */
    public static function loadAssets ($options = array()) {
        template::setJs('/js/jquery.textarea.js');
        if (config::getModuleIni('content_article_jquery_markedit')){
            moduleLoader::includeTemplateCommon('jquery-markedit');
            jquery_markedit_load_assets($options);
        }

    }

    
    /**
     * method for validating user input
     */    
    public function validate(){
        if (empty($_POST['submit'])){
            return;
        }
        if (strlen($_POST['title']) < 1){
            $this->errors['title'] = lang::translate('content_validate_error_title');
        }

        if (!isset($_POST['disable_headline'])) {
            $_POST['disable_headline'] = 0;
        } else {
            $_POST['disable_headline'] = 1;
        }
    }

    /**
     * method for getting a cached article.
     * if no article is cached we get a fresh article.  
     * @param int $id id of the article
     * @return array $article the article as a row. 
     */
    public function getCachedArticle ($id){
        $article_row = cache::get(self::$reference, $id);
        if (!$article_row) {
            $article_row = $this->getArticle();
            cache::set(self::$reference, $id, $article_row);
        }
        return $article_row;
    }

    /**
     * method for santizing a submission
     * Encodes special chars. 
     */
    public function sanitize(){
        $_POST = html::specialEncode($_POST);
    }

    /**
     * method for getting current article.
     * This means article with id of /content/article/view/{$id}
     *
     * @param   string  $state 'update' or else
     * @return  array   $article row containing the selected article
     */
    public static function getArticle($state = NULL){     
        $db = new db();
        if (empty(self::$articleRaw)){        
            $row  = $db->selectOne('content_article', 'id', self::$id);      
            self::$articleRaw = $row;

            if (self::$articleRaw == NULL) {           
                session::setActionMessage(lang::translate('content_does_not_exists_anymore'));
                http::locationHeader("/content/index");
                die;
            }
        }

        if (empty(self::$articleRaw)){
            return false;
        }
        
        if ($state == 'update') {
            return self::$articleRaw;
        } else {
            return self::filterArticle(self::$articleRaw);
        }
    }

    /**
     * method for getting an article from an id
     *
     * @param   int     $id id of article
     * @return  array   $row article row
     */
    public static function getArticleFromId($id){
        $db = new db ();
        $row = $db->selectOne('content_article', 'id', $id);
        return $row;
    }

    /**
     * method for getting a article title from id
     *
     * @param   int     $id id of article
     * @return  string  $str title of article or constant
     *                  which tells that article has not a title. 
     */
    public static function getTitleFromId($id){
        $db = new db ();
        $search = array ('id' => $id);
        $row = $db->selectOne('content_article', array ('title'), $search);
        
        if (isset($row['title'])){
            return $row['title'];
        }
        return lang::translate('content_article_has_not_title');
    }

    /**
     * method for getting current users info
     *
     * @return array $account row containing the selected users cred
     */
    public static function getUserInfo($userId){
        $db = new db();
        $row = $db->selectOne('account', 'id', $userId);
        return $row;
    }



    /**
     * method for getting latest articles
     *
     * @param   int $from from
     * @param   int $limit limit
     * @return  array $rows rows of latest articles. 
     */
    public static function getRowsRSS($from = 0, $limit = 20){

            $rows = dbQ::setSelect('content_article', 'id, title, abstract, created')->
                    limit($from, $limit)->
                    fetch();

            foreach ($rows as $key => $val){
                $url = contentArticle::getArticleUrl($val['id'], $val['title']);
                $url = html::getUrl($url);
                $rows[$key]['url'] = $url; 
            }
            return $rows;
    }


    /**
     * method for getting latest articles left joined with account
     *
     * @param   int $from from 
     * @param   int $limit limit
     * @return  array $rows latest article rows joined with user info
     */
    public function getLatest($from = 0, $limit = 20){

            $rows = dbQ::setSelect('content_article', 'id, title, user_id, abstract, created')->
                    limit($from, $limit)->
                    fetch();
            return $rows;
    }
    
    public static function sphinxHit ($id) {
        
        $row = dbQ::setSelect('content_article', 'id, title, user_id, abstract, created')->
                filter('id =', $id)->
                fetchSingle();
        if (empty($row)) return '';
        viewContentArticle::articleListSingle($row);
        
    }

    /**
     * method for filtering an article for output
     *
     * @param array     $raw article row raw
     * @return array    $filtered article row filtered
     */
    public static function filterArticle($article){
        
        static $filtered_article = null;

        if (!$filtered_article){
            $filters = config::getModuleIni('content_article_filters');
            $article['article'] = moduleLoader::getFilteredContent($filters, $article['article']);
            $filtered_article = $article;
            return $article;
        }
        return $filtered_article;
    }


    /**
     * method for adding an article to database
     *
     * @return  boolean $res true on succes false on failure
     */
    public static function createArticle (){
        $db = new db();
        db::$dbh->beginTransaction();
        
        // not used, but may be used at some point
        if (session::isAdmin()){
            $published = 1;
        } else {
            $published = 0;
        }

        // decode again and set some global $_POST fields
        $_POST = html::specialDecode($_POST);
        $_POST['published'] = $published;
        $_POST['user_id'] = $_SESSION['id'];
        $_POST['created'] = date('Y-m-d H:i:s');

        try {            
            $values = db::prepareToPostArray(self::$post);            
            $res = $db->insert('content_article', $values);
            $last_insert_id = db::$dbh->lastInsertId();
            
            // manipulate system menu
            if (isset($_POST['is_menu'])) {
                $values['id'] = $last_insert_id;
                $values['pid'] = 0;
                $menu = menu::getSystemMenuArray(self::$reference);
                menu::createSystemMenuItem($menu, $values);
                menu::updateSystemMenuArray($menu, self::$reference);
            }
           
            // create params for events that may be triggered
            $event_params = array (
                'action' => 'insert',
                'reference' => self::$reference, 
                'parent_id' => $last_insert_id);
            
            // trigger events
            event::triggerEvent(
                config::getModuleIni('content_article_events'),
                $event_params
            );
        } catch (PDOException $e) {            
            db::$dbh->rollBack();
            cos_error_log($e->getTraceAsString());
            return false;
        }
        // ok
        $res = db::$dbh->commit();
        return $res;
    }

    /**
     * method for deleting current article. Deletes from uri. 
     *
     * @return boolean $res true on success false on failure
     */
    public function deleteArticle(){

        $id = uri::$fragments[3];
        $db = new db();
        
        //if (get_module_ini('content_article_jquery_sort')){
            $ary = menu::getSystemMenuArray('content/article');
            $ret = menu::menuItemHasChildren($ary, $id);
            if ($ret){
                self::$errors['title'] = lang::translate('content_article_has_sub_menu_items');
                return false;
            }
        //}

        db::$dbh->beginTransaction();
        try {
                
            $ary = menu::getSystemMenuArray(self::$reference);

            // work on ary menu per reference
            menu::deleteMenuItem($ary, null, $id);
            menu::updateSystemMenuArray($ary, self::$reference);

            cache::delete(self::$reference, $id);
            $res = $db->delete('content_article', 'id',  $id);

            $event_params = array (
                'action' => 'delete',
                'reference' => self::$reference, 
                'parent_id' => $id);
            
            event::triggerEvent(
                config::getModuleIni('content_article_crud_events'),
                $event_params);
            
        } catch (PDOException $e) {
            error_log($e->getTraceAsString());
            db::$dbh->rollBack();
            return false;
        }

        $res = db::$dbh->commit();
        return $res;

    }

    /**
     * method for updating an article
     *
     * @return boolean $res true on success and false on failure.
     */
    public function updateArticle (){
        include_once "coslib/cache.php";

        $id = uri::$fragments[3];
        $db = new db();
        db::$dbh->beginTransaction();

        $_POST = html::specialDecode($_POST);
        $_POST['updated'] = date('Y-m-d H:i:s');
        
        // check that we are not removing child menus
        if ($this->isMenuItem($id) && !isset($_POST['is_menu'])){
            $ary = menu::getSystemMenuArray('content/article');
            $ret = menu::menuItemHasChildren($ary, $id);
            if ($ret){
                $this->errors['title'] = lang::translate('content_article_has_sub_menu_items_form');
                return false;
            }
        }
        
        try {
            // remove frontpages if new fronpage is set.
            self::removeFrontPages();
            
            $event_params = array (
                'action' => 'update',
                'reference' => self::$reference, 
                'parent_id' => $id);
            
            event::triggerEvent(
                config::getModuleIni('content_article_events'),
                $event_params);
                        
            unset(contentArticle::$post['created']);
            $values = db::prepareToPostArray(contentArticle::$post);   
            unset($values['created']);

            if ($this->isMenuItem($id) ){
                if (isset($_POST['is_menu'])) {
                    // update system menu
                    $ary = menu::getSystemMenuArray(self::$reference);
                    menu::updateMenuItem($ary, $values, $id);
                    menu::updateSystemMenuArray($ary, self::$reference);
                } else {
                    // delete menu item
                    $ary = menu::getSystemMenuArray(self::$reference);
                    // work on ary menu per reference
                    menu::deleteMenuItem($ary, null, $id);
                    menu::updateSystemMenuArray($ary, self::$reference);
                }
            } else {
                // add new menu item
                if (isset($_POST['is_menu'])) {
                    $values['id'] = $id;
                    $values['pid'] = 0;
                    $menu = menu::getSystemMenuArray(self::$reference);
                    menu::createSystemMenuItem($menu, $values);
                    menu::updateSystemMenuArray($menu, self::$reference);
                    unset($values['pid']);
                    
                } 
            }
            // delete cache
            cache::delete(self::$reference, $id);
            
            // update article
            $res = $db->update('content_article', $values, $id);
            

        } catch (PDOException $e) {
            db::$dbh->rollBack();
            cos_error_log($e->getTraceAsString());
            return false;
        }

        $res = db::$dbh->commit();
        return $res;
    }
    
    public function isMenuItem ($id) {
        $row = self::getArticleFromId($id);
        if ($row['is_menu'] == 1) {
            return true;
        }
        return false;
    }



    /**
     *
     * @param  int      $id id of article
     * @param  string   $title title of article
     * @param  string   $extra extra is added if you want to create anchors 
     *                         for the url. 
     * @return string   string containing url path to artcile
     */
    public static function getArticleUrl($id = null, $title = null, $options = null){
        if (isset($options['edit_link'])) {
            $url = "/content/article/edit/$id";
        } else {       
            $url = strings::utf8Slug("/content/article/view/$id", $title);
        }
        return $url;
    }

    /**
     * creates a normal article url
     * 
     * @param int $id of article
     * @return string the article url
     */
    public static function getArticleUrlReal($id){
        $url = "/content/article/view/$id";
        return $url;
    }

    

    /**
     * method for getting a html link to current article.
     *
     * @return string HTML link to current article
     */
    public function getArticleHTMLLink(){
        $article = $this->getArticle();
        //$url = "/content/article/view/" . self::$id;
        $url = self::getArticleUrl($article['id'], $article['title']);
        return html::createLink($url, html::specialEncode($article['title']));
    }


    /**
     * method for getting the title for a article
     * @return string
     */
    public function getArticleTitle(){
        $article = $this->getArticle();
        $title = $article['title'] . MENU_SUB_SEPARATOR . lang::translate('content_article_view_article');
        return $title;
    }



    /**
     * recursive method for getting article tree.
     *
     * @param       int     id of article. Then we find parent_id and id of that ... etc
     * @return      array   the static var will be returned at last.
     */
    public function getArticleTreeJQuery(){
        $str = "<ul id=\"menu\">\n";
        $menu = menu::getSystemMenuArray('content/article');

        foreach ($menu as $val){
            
            if (empty($val['sub']) || session::isAdmin()){
                $str.= "<li>\n";
               $str.= html::createLink(
                    "/content/article/view/$val[id]",
                    $val['title']);
            } else {
                $str.= "<li class=\"menu\">\n";
               $str.= $val['title'];
            }
            
            if (!empty($val['sub'])){
                
                $str.="<ul>\n";
                foreach ($val['sub'] as $sub_val){
                    $str.= "<li>" . html::createLink("/content/article/view/$sub_val[id]", $sub_val['title']) . "</li>\n";
                }
                $str.="</ul>\n";
            }
            $str.="</li>\n";
            
        }

        $str.= layout::parseMainMenuList();
        $str.="</ul>";
        return $str;
    }
    
    /**
     * recursive method for getting article tree as an array.
     * @param       int     id of article. Then we find parent_id and id of that ... etc
     * @return      array   the static var will be returned at last.
     */
    public function getArticleTree($id = 0){
        $search = array('parent_id' => $id);
        $db = new db();
        $rows = $db->selectAll(
            'content_article',
            array( 'id, parent_id, title', 'weight'), $search, null, null, 'weight');

        if(empty($rows)) return array();
        $out = array();
        foreach($rows as $val){
            $val['url'] = self::getArticleUrl($val['id'], $val['title']);
            $val['sub'] = $this->getArticleTree($val['id']);
            if (empty($val['sub'])) {
                unset($val['sub']);
            }

            $out[] = $val;
        }
        return $out;
    }

   /**
    * method for getting an article trail
    *
    * @param   array     the trail array to parse
    * @return  string    html string with trail of links
    */
    function getArticleTrailHtml(){
        $db = new db();

        $stack = menu::getStack(self::$reference, self::$id);
        $i = count($stack);
        $str = '';
        $use_bc = config::getModuleIni('content_article_crumb');

        foreach ($stack as $key => $val){    
            $i--;
            $row = $db->selectOne('content_article', 'id', $key, array('id', 'title'));
            $url = self::getArticleUrl($row['id'], $row['title']);
            if ($i){
                if ($use_bc){
                    $str.= html::createLink($url, html::specialEncode($row['title'])) . " :: ";
                }
            } else {
                $str.= html::specialEncode($row['title']);
            }            
        }
        return $str;
    }


    /**
     * check if a new front page has been set. 
     * @return mixed  $res result of query or if no query void
     */
    public static function removeFrontpages (){
        $db = new db();
        if (!isset($_POST['is_frontpage'])) {
            $_POST['is_frontpage'] = 0;
        } else {
            $res = $db->rawQuery("UPDATE content_article SET is_frontpage = '0'");
            $_POST['is_frontpage'] = 1;
            return $res;
        }
    }

    /**
     * get all num article rows
     * @return int $num_rows number of articles
     */
    public function getNumRows() {
        $db = new db();
        return $db->getNumRows('content_article');
    }

   /**
    * get db row for redirecting
    * @return int $id article it.
    */
    public function getRedirectRow (){
        $db = new db();
        $row = $db->selectOne('content_article', null, array('is_frontpage' => 1), array('id', 'title'));
        return $row;
    }

    /** 
     * get a URL from article id
     * @param int $id article id
     * @return string $url article url from id.
     */
    public function getRedirect ($id){

        $title = self::getTitleFromId($id);
        $url = self::getArticleUrl($id, $title);
        //die;
        return $url;
    }

    /**
     * method for getting a article link from a id. 
     * @param int $id article id
     * @param string $extra attach to URL
     * @return string $link html link
     */
    public static function getLinkFromId ($id, $options = null){
        $row = self::getArticleFromId($id);
        if (empty($row)) {
            return lang::translate('content_article_reference_deleted');
        }
        $title = self::getTitleFromId($id);
        $url = self::getArticleUrl($id, $title, $options);
        return html::createLink($url, $title, $options);
    }

    /**
     * method for returning article id from URL
     * @return int $id return id from url
     */
    public static function getIdFromUrl(){
        if (!isset(uri::$fragments[3])) return false;
        return filter_var(uri::$fragments[3], FILTER_VALIDATE_INT, array("min_range"=>1));
    }
    
    /**
     * fetch tags and displays them in /content/article/tags/id
     * @return type 
     */
    public static function tagController (){

        moduleLoader::includeModule('tags');
        $id = self::getIdFromUrl();
        
        $headline = lang::translate('content_tags_list_entries_title');

        $tag = tags::getTagSingle($id);
        if (!$tag) {
            echo lang::translate('content_article_no_such_tags');
            return;
        }
        
        $url = strings::utf8Slug("/content/article/tags/$tag[id]", $tag['title']);
        http::permMovedHeader($url);

        $headline.= MENU_SUB_SEPARATOR_SEC;
        $headline.= $tag['title'];
        
        $title = lang::translate('content_tags_page_tag_title');
        $title.= MENU_SUB_SEPARATOR;
        $title.= $tag['title'];

        template::setTitle($title);       
        headline_message($headline);

        $references = tags::getAllReferenceTag (self::$reference, $id);
        $num_rows = tags::getAllReferenceTagNumRows(self::$reference, $id);

        if ($num_rows == 0){
            echo lang::translate('content_article_tags_no_articles');
            return;
        }

        include_once "pearPager.php";

        $per_page = config::getModuleIni('content_article_per_page');
        $pager = new pearPager($num_rows, $per_page);
        $rows = dbQ::setSelect('content_article')->
            filterIn("id IN ", $references)->
            order('id', 'DESC')->
            limit($pager->from, $per_page)->        
            fetch();
        
        viewContentArticle::articleList($rows);
        $pager->pearPage();
    }
}