<?php

/**
 * view file for content/article
 *
 * @package content
 */

class viewContentArticle {
    /**
     * function for creating a form for insert, update and deleting articles. 
     *
     *
     * @param string    method (update, delete or insert)
     * @param int       id (if delete or update)
     */
    public static function articleForm($method, $id = null, $values = array()){
        if ($method == 'insert'){
            templateView::includeModuleView('content', 'article_add');
        }

        if ($method == 'update'){
            $values =  contentArticle::getArticleFromId($id);
            $values = html::specialEncode($values);
            templateView::includeModuleView('content', 'article_update', $values);
        }

        if ($method == 'delete'){
            templateView::includeModuleView('content', 'article_delete');
        }
    }


    /**
     * print article info to screen
     * @param array $article
     */
    public static function header ($user_id, $created){
        if (config::getModuleIni('content_article_hide_user_info')) {
            return '';
        }

        $date = time::getDateString($created);        
        echo user::getProfile($user_id, $date);
    }

    /**
     * function for viewing a single article
     *
     * @param array     article row
     * @param array>    category row
     */
    public static function article($article) {

        $events = event::getTriggerEvent(
            config::getModuleIni('content_article_events'), 
                array(
                    'action' => 'get',
                    'reference' => contentArticle::$reference,
                    'parent_id' => $article['id'])
        );
        
        $view_modules = config::getModuleIni('content_view_sub_modules');
        moduleLoader::includeModules($view_modules);
        
        $return_url = contentArticle::getArticleUrl($article['id'], $article['title']);
        $return_url = html::getUrl($return_url);
        
        $options = array (
            'parent_id' => $article['id'],
            'reference' => 'content/article',
            'return_url' => $return_url,
            'entity' => $article
        );

        $ary = array ();        
        $ary = moduleLoader::subModuleGetPreContent($view_modules, $options);
        $ary = array_merge($events, $ary);
        
        array_unshift($ary, self::getAdminLinks($article['id']));

        echo implode("<hr />\n", $ary);
        
        $ary = moduleLoader::subModuleGetInlineContent($view_modules, $options);

        echo implode("<hr />\n", $ary);
        
        echo "<div id=\"toc\"></div>\n";
        echo "<div id=\"content_article\">\n";
        echo $article['article'];
        echo "</div>\n";
        
        $ary = moduleLoader::subModuleGetPostContent($view_modules, $options);
        echo implode("<hr />\n", $ary);
    }


    

    /**
     * function for printing an article list to screen.
     *
     * @param   array article rows
     */
    public static function articleList($articles) {
        foreach($articles as $article){
            self::articleListSingle ($article);
        }
    }
    
    public static function articleListSingle ($article) {
        $url = contentArticle::getArticleUrl($article['id'], $article['title']);
        echo "<article>\n";
        echo "<header>\n";
        echo headline_message(html::createLink($url, html::specialEncode($article['title'])));
        self::header($article['user_id'] , $article['created']);

        event::triggerEvent(
            config::getModuleIni('content_article_events'), 
                array(
                    'action' => 'view',
                    'reference' => contentArticle::$reference,
                    'parent_id' => $article['id'])
         );
         echo "</header>\n";
         echo "<hr />\n";
         
         $abstract = trim($article['abstract']);
         if (empty($abstract)){
            $article['abstract'] = '...';
         }
         echo html::specialEncode($article['abstract']);
         echo "<hr />\n";
         echo "</article>\n";
    }



    /**
     * function for viewing admin options on article (Edit, delete, create sub page)
     * @param   int  article_id
     */
    public static function getAdminLinks($id){
        $str = '';
        if (session::isAdmin()){
            $str.= lang::translate('content_article_admin_options') . MENU_SUB_SEPARATOR_SEC;
            $ary = array();

            // default content/article options: edit and delete
            $ary[] = html::createLink(
                    "/content/article/edit/$id",
                    lang::translate('content_article_edit')
            );

            $ary[] = html::createLink(
                    "/content/article/delete/$id",
                    lang::translate('content_article_delete')
            );

            $str.=self::getOptionLinks($ary);
            // "<hr />\n";
        }
        return $str;
    }

    public static function getOptionLinks ($ary = array()){
        $num = count($ary);
        $ret_str = '';
        foreach ($ary as $val){
            $num--;
            if ($num) {
                $ret_str.= $val . MENU_SUB_SEPARATOR;
            } else {
                $ret_str.= $val;
            }
        }
        return $ret_str;
    }
}
